#!/usr/bin/perl -w

# This script makes all edits needed when post-processing the output of
# xsltproc on the SRD spell text.  Since we're processing all files
# anyway, we go ahead and build the list of macros that will be needed
# by the most common spellcasting classes for D&D 3.0E, 3.5E, and PF.
# Basically this means a list of macros for bards, clerics, druids,
# paladins, rangers, sorcerors, and wizards.  This is done by creating a
# file that runs a script which produces the correct XML for a MapTool
# macroset's "content.xml" file.

# Lines that need to be cleaned up are these:
#
# _CSS_LINK
# _MAGIC_LINK
# <link rel="stylesheet" type="text/css" href="_CSS@[r: getMacroLocation()]"/>
# <br clear="all"/>

# For the spell list creation, these lines are the needed ones:
# <h1>Jump<small>
# <th><span class="link">Level</span>:</th><td>Drd 1, Rgr 1, Sor/Wiz 1</td>
use Data::Dumper;
my($spell);	# Current spell name (from parsing the file)
my(%SpellNames); # List of all macro names => spell names (for xref)
my(%bySpell);	# List of all macro names => filenames (for Lib: token)
my(%byClass);	# Macro names per class/level (for players to copy from)

print "#!/usr/bin/perl -w\n\n";

foreach $ARGV (@ARGV) {
    open(STDIN, $ARGV) or die "Can't open file '$ARGV'!\n";
    $spell = '-';
    while (<STDIN>) {
	/<h1>/ and /<h1>(.*?)</ and $spell = $1;
	/>Level</ and /<td>(.*?)</ and &addToList($spell, $1);
    }
}
&printHeader;
$Data::Dumper::Indent = 1;
$Data::Dumper::Sortkeys = 1;
print Data::Dumper->Dump([  \%byClass, \%bySpell, \%SpellNames ],
			 [qw(*byClass   *bySpell   *SpellNames)] );
&printFooter;
exit 0;

sub addToList {
    my($spell, $levels) = @_;
    my(@list) = ();

    $spell = &Unicode::replace($spell);
    my $macname = $spell;
    $macname =~ s/\W+//g;
    $SpellNames{ $macname } = $spell;
    $bySpell{ $macname } = $ARGV;

    foreach my $field (split(/\s*,\s*/, $levels)) {
	($classes, $level) = split(/\s+/, $field);
	$classes = lc($classes);
	foreach my $class (split('/', $classes)) {
	    push(@{$byClass{ $class }[ $level ]}, $macname);
	    push(@list, "$class $level");
	}
    }
    print "# $spell: ", join(", ", sort @list), "\n";
}

sub printHeader {
    print << 'End_Of_Header';
#!/usr/bin/perl -w

&setup;
my %Ability = (
      'brd' => 'Cha',
      'sor' => 'Cha',
      'wiz' => 'Int',
);

print << 'EOF';
#!/bin/bash

function startNewFile {
    if (( index != 0 )); then
	echo "</macroPropertiesMap>"
	echo 1>&2
    fi

    if (( $# > 0 )); then
	index=0
	echo "Starting '$1' ..." 1>&2
	exec > "$1"
	echo "<macroPropertiesMap>"
    fi
}

function encode {
    sed -e 's/</\&lt;/g' -e 's/>/\&gt;/g' \
	-e 's/"/\&quot;/g' -e "s/'/\&apos;/g" \
	"$@"
}

function spellMacros {
    sed \
EOF
foreach my $macname (sort keys %bySpell) {
    my $filename = $bySpell{ $macname };
    $filename =~ s#.*/##;
    $filename =~ s#[.].*?$##;
    print "\t-e 's!_SPELL_NAME(/srd/spells/$filename.htm)!$macname\@Lib:35PFSpells!g' \\\n";
}
print << 'EOF';
	"$@"
}

# Parameters:  macroName fileName spellName
#   macroName is the spell name with all non-\w characters removed,
#   fileName is the filename from the 4-macro/ directory, and
#   spellName is the actual spell name with all punctuation included.
function libMacro {
    macname=$1 ; shift
    filename=$1 ; shift
    spell="$*"
    let index=index+1
    echo -ne "\r  $index" 1>&2
    cat <<__EOM__
<entry>
  <int>$index</int>
  <net.rptools.maptool.model.MacroButtonProperties>
    <saveLocation>Token</saveLocation>
    <index>$index</index>
    <colorKey>default</colorKey>
    <hotKey>None</hotKey>
    <command>$(spellMacros "$filename" | encode)
</command>
    <label>$macname</label>
    <group>$GROUP</group>
    <sortby></sortby>
    <autoExecute>true</autoExecute>
    <includeLabel>false</includeLabel>
    <applyToTokens>true</applyToTokens>
    <fontColorKey>black</fontColorKey>
    <fontSize>1.00em</fontSize>
    <minWidth></minWidth>
    <maxWidth></maxWidth>
    <allowPlayerEdits>false</allowPlayerEdits>
    <toolTip>$(echo "$spell" | encode)</toolTip>
    <commonMacro>false</commonMacro>
    <compareGroup>true</compareGroup>
    <compareSortPrefix>true</compareSortPrefix>
    <compareCommand>true</compareCommand>
    <compareIncludeLabel>true</compareIncludeLabel>
    <compareAutoExecute>true</compareAutoExecute>
    <compareApplyToSelectedTokens>true</compareApplyToSelectedTokens>
  </net.rptools.maptool.model.MacroButtonProperties>
</entry>
__EOM__
}

function selectivelyCopyMacros {
    macname="Selectively copy these macros"
    let index=index+1
    echo -ne "\r  $index" 1>&2
    macro="[macro('_selectiveCopy@Lib:35PFSpells'): '' ]
"
    cat <<__EOM__
<entry>
  <int>$index</int>
  <net.rptools.maptool.model.MacroButtonProperties>
    <saveLocation>Token</saveLocation>
    <index>$index</index>
    <colorKey>cyan</colorKey>
    <hotKey>None</hotKey>
    <command>$(echo "$macro" | encode)
</command>
    <label>$macname</label>
    <group>Configuration</group>
    <sortby></sortby>
    <autoExecute>true</autoExecute>
    <includeLabel>false</includeLabel>
    <applyToTokens>true</applyToTokens>
    <fontColorKey>black</fontColorKey>
    <fontSize>1.00em</fontSize>
    <minWidth></minWidth>
    <maxWidth></maxWidth>
    <allowPlayerEdits>false</allowPlayerEdits>
    <toolTip>Interactively copy selected macros.</toolTip>
    <commonMacro>false</commonMacro>
    <compareGroup>true</compareGroup>
    <compareSortPrefix>true</compareSortPrefix>
    <compareCommand>true</compareCommand>
    <compareIncludeLabel>true</compareIncludeLabel>
    <compareAutoExecute>true</compareAutoExecute>
    <compareApplyToSelectedTokens>true</compareApplyToSelectedTokens>
  </net.rptools.maptool.model.MacroButtonProperties>
</entry>
__EOM__
}

# Parameters:  className abilityName spellLevel macroName spellName
#   className is the typical three-letter abbreviation or full domain name,
#   abilityName is 'Int', 'Wis', or 'Cha',
#   spellLevel is numeric, from 0..10 (for extended spell lists),
#   macroName is the spell name with all non-\w characters removed,
#   spellName is the actual spell name with all punctuation included.
function playerMacro {
    class=$1 ; shift
    ability=$1 ; shift
    level=$1 ; shift
    macname=$1 ; shift
    spell="$*"
    let index=index+1
    echo -ne "\r  $index" 1>&2
    macro="[ params = json.set('{}', 'Name', '$macname', 'Type', 'Spell', 'SpellLevel', $level, 'Ability', '$ability', 'CasterLevel', Level) ]
[ dnd.execMacro(params) ]
[ abort(0) ]
"
    cat <<__EOM__
<entry>
  <int>$index</int>
  <net.rptools.maptool.model.MacroButtonProperties>
    <saveLocation>Token</saveLocation>
    <index>$index</index>
    <colorKey>default</colorKey>
    <hotKey>None</hotKey>
    <command>$(echo "$macro" | encode)
</command>
    <label>$macname</label>
    <group>Spell Level $level</group>
    <sortby></sortby>
    <autoExecute>true</autoExecute>
    <includeLabel>false</includeLabel>
    <applyToTokens>true</applyToTokens>
    <fontColorKey>black</fontColorKey>
    <fontSize>1.00em</fontSize>
    <minWidth></minWidth>
    <maxWidth></maxWidth>
    <allowPlayerEdits>false</allowPlayerEdits>
    <toolTip>$spell (level $level) for $class</toolTip>
    <commonMacro>false</commonMacro>
    <compareGroup>true</compareGroup>
    <compareSortPrefix>true</compareSortPrefix>
    <compareCommand>true</compareCommand>
    <compareIncludeLabel>true</compareIncludeLabel>
    <compareAutoExecute>true</compareAutoExecute>
    <compareApplyToSelectedTokens>true</compareApplyToSelectedTokens>
  </net.rptools.maptool.model.MacroButtonProperties>
</entry>
__EOM__
}

# This doesn't really do anything (no macros needed), but it triggers
# the copying of the README.rptok token so that it appears in the
# directory with all of the other tokens. :)
startNewFile README.xml
  DIR=../Configuration
  GROUP=Configuration
  index=1	# Forces the closing tag for <macroPropertiesMap>

startNewFile lib_spells.xml
  DIR=../Configuration
  GROUP=Configuration
  libMacro onCampaignLoad $DIR/onCampaignLoad@Lib:Spells.mtm "Autodefine functions"
  libMacro _CSS $DIR/CSS@Lib:Spells.mtm "CSS sheet"
  libMacro _CalcDC $DIR/CalcDC@Lib:Spells.mtm "Calculates DC given SL and CL"
  libMacro _diceBox $DIR/DiceBox@Lib:Spells.mtm "Evaluates dice rolls"
  libMacro _execMacro $DIR/execMacro@Lib:Spells.mtm "Execution front-end"
  libMacro _selectiveCopy $DIR/selectiveCopy@Lib:Spells.mtm "Selective copy"
  libMacro _SLA $DIR/Type_SLA@Lib:Spells.mtm "Spell-Like Ability rules summary"
  libMacro _Spell $DIR/Type_Spell@Lib:Spells.mtm "Spell Ability rules summary"
  libMacro _Su $DIR/Type_Su@Lib:Spells.mtm "Supernatural Ability rules summary"

  GROUP=Spells
EOF
foreach my $macname (sort keys %bySpell) {
    my $spell = $SpellNames{ $macname };
    print "  libMacro $macname $bySpell{ $macname } \Q$spell\E\n";
}
my %longName = (
    'brd' => 'Class Bard',
    'clr' => 'Class Cleric',
    'drd' => 'Class Druid',
    'pal' => 'Class Paladin',
    'rgr' => 'Class Ranger',
    'sor' => 'Class Sorceror',
    'wiz' => 'Class Wizard',
);
my @keys = sort keys %byClass;
foreach my $class (@keys) {
    $longName{$class} = "Domain \u$class" unless exists $longName{$class};
}
foreach my $class (@keys) {
    my $longClassName = $longName{$class};
    my $ability = exists $Ability{ $class } ? $Ability{ $class } : "Wis";
    print "\n";
    print "startNewFile '$longClassName.xml'\n";
    print "  selectivelyCopyMacros\n";
    foreach my $level (0..9) {
	my $ref = $byClass{$class}[$level];
	next unless defined $ref;

	foreach my $macname (@$ref) {
	    my $spell = $SpellNames{ $macname };
	    print "  playerMacro '$longClassName' $ability $level $macname \Q$spell\E\n";
	}
    }
}
print "startNewFile\n";
exit 0;

sub setup {
End_Of_Header
}

sub printFooter {
    print << 'End_Of_Footer';
}
End_Of_Footer
}

package Unicode;

# This subroutine reads a bunch of text that uses the '\u' notation or
# '&#x' HTML notation and converts them to regular ASCII text using a
# conversion table.

BEGIN {
    $UNICODE = '\u';
    $HTML = '&#x';
    %unicode = (
	'00b0' => '&deg;',
	'00bd' => '&frac12;',
	'00d7' => '&times;',
	'2014' => ' -- ',
	'2019' => "'",
	'201c' => '<span class="term">',
	'201d' => '</span>',
    );
    $D = '[[:xdigit:]]';
};

sub replace {
    local($_) = @_;
    s/\Q$UNICODE\E($D{4})/$unicode{$1}/g;
    s/\Q$HTML\E($D{1,4});/$unicode{$1}/g;
    $_;
}
