#!/usr/bin/perl -w

# This script reads the files given on the command line and creates a
# list of properties for MapTool.  The files to be processed are
# expected to those in the "2-macros/" subdirectory, ie. they've already
# been processed initially and the data is in a well-known format.
#
# Each property name is of the form '[A-Z]_URL' and corresponds to the
# first letter of the spell name.  It generates a JSON object which has
# the name of the spell as the field name and the base URL name as the
# value.  The base has to have the URL prefix and the ".html" suffix
# added to be useful.  The MapTool code will treat the URL as fully
# qualified if it contains a colon (":") and as a relative URL if it
# starts with a slash ("/").  This allows a prefix to be added to the
# URLs easily.

# Here's what the XML for a property looks like.  The first two elements
# are "overhead" and the individual <entry> elements are properties.
#
#  <propertyMap class="net.rptools.CaseInsensitiveHashMap">
#    <store>
#      <!-- One property... -->
#      <entry>
#        <string>current_suit</string>
#        <net.rptools.CaseInsensitiveHashMap_-KeyValue>
#          <key>current_suit</key>
#          <value class="string">[9,3,5,8,4,1,2,6,7]</value>
#          <outer-class reference="../../../.." />
#        </net.rptools.CaseInsensitiveHashMap_-KeyValue>
#      </entry>
#    </store>
#  </propertyMap>

my %Exception = ();

sub url {
    local($_) = @_;
    s/(\w+)/\u$1/g;	# Capitalize the first letter of each word
    s/\W+//g;		# Remove all spaces, punctuation, etc.
    if (exists $Exception{$_}) {
	return $Exception{ $_ };	# Done like this for brkpts
    }
    "\l$_";		# Change first letter to lowercase
}

# Take out commas since they can't exist inside a string list
# property...
sub listEntry {
    local($_) = @_;
    my $url;

    # This fixes 'Mass', 'Lesser', and 'Greater' spells.
    # Can't automate the process with spells containing proper names.
    if (/, (\w+)/) {
	$url = "\L$1\ESpellName";
    } else {
	$url = &url($_);
    }
    s/[^\w\s]+//g;
    '"' . $_ . '":"' . $url . '"';
}
sub entryTemplate {
    my ($prop, @values) = @_;
    die "Illegal property name '$prop'." if $prop !~ /^\w$/;

    my $letter;
    $letter = "\l$prop";
    my $value = join(", ", map { &listEntry($_) } @values);
    return
"<entry>
    <string>${prop}_URL</string>
    <net.rptools.CaseInsensitiveHashMap_-KeyValue>
	<key>${letter}_url</key>
        <value class='string'>{ $value }</value>
	<outer-class reference='../../../..' />
    </net.rptools.CaseInsensitiveHashMap_-KeyValue>
</entry>\n";
}

&loadExceptionTable();

my $cacheEntry = "A";
my @list = ();
my $spellname;
while (<>) {
    next unless /<h1>/;
    ($spellname) = /<h1>(.*?)<small>/;
    if ($spellname !~ /^$cacheEntry/i) {
#	print "@list";
	print &entryTemplate($cacheEntry, @list);
	$cacheEntry = substr($spellname, 0, 1);
	@list = ();
    }
    $spellname =~ s/&amp;/\&/g;
    $spellname =~ s/&apos;/'/g;
    push(@list, $spellname);
}
if (@list > 0) {
#   print "@list";
    print &entryTemplate($cacheEntry, @list);
}

sub loadExceptionTable {
    while (<DATA>) {
	next if /^#/;
	chomp;
	@F = split();
	my $base = pop(@F);
	$base =~ s#/srd/spells/(.*)\.html#$1#;
	my $name = join("", @F);
	$Exception{ $name } = $base;
    }
    return;
}

__END__
#Arcane Sight Greater             /srd/spells/greaterSpellName.html
#Bears Endurance Mass             /srd/spells/massSpellName.html
Clenched Fist                     /srd/spells/clenchedFist.html
Crushing Hand                     /srd/spells/crushingHand.html
Forceful Hand                     /srd/spells/forcefulHand.html
Grasping Hand                     /srd/spells/graspingHand.html
Interposing Hand                  /srd/spells/interposingHand.html
Bigbys Clenched Fist              /srd/spells/clenchedFist.html
Bigbys Crushing Hand              /srd/spells/crushingHand.html
Bigbys Forceful Hand              /srd/spells/forcefulHand.html
Bigbys Grasping Hand              /srd/spells/graspingHand.html
Bigbys Interposing Hand           /srd/spells/interposingHand.html
Blindness_Deafness                /srd/spells/blindnessDeafness.html
#Bulls Strength Mass              /srd/spells/massSpellName.html
#Cats Grace Mass                  /srd/spells/massSpellName.html
#Charm Monster Mass               /srd/spells/massSpellName.html
Clairaudience_Clairvoyance        /srd/spells/clairaudienceClairvoyance.html
#Command Greater                  /srd/spells/greaterSpellName.html
#Corrupt Weapon                    ??????????????????????????????
#Cure Critical Wounds Mass        /srd/spells/massSpellName.html
#Cure Light Wounds Mass           /srd/spells/massSpellName.html
#Cure Moderate Wounds Mass        /srd/spells/massSpellName.html
#Cure Serious Wounds Mass         /srd/spells/massSpellName.html
Disintegrate                      /srd/spells/disintegrate.html
#Dispel Magic Greater             /srd/spells/greaterSpellName.html
Instant Summons                   /srd/spells/instantSummons.html
Drawmijs Instant Summons          /srd/spells/instantSummons.html
#Eagles Splendor Mass             /srd/spells/massSpellName.html
#Enlarge Person Mass              /srd/spells/massSpellName.html
Black Tentacles                   /srd/spells/blackTentacles.html
Evards Black Tentacles            /srd/spells/blackTentacles.html
#Foxs Cunning Mass                /srd/spells/massSpellName.html
#Geas Lesser                      /srd/spells/lesserSpellName.html
Geas_Quest                        /srd/spells/geasQuest.html
#Globe of Invulnerability Lesser  /srd/spells/lesserSpellName.html
#Glyph of Warding Greater         /srd/spells/greaterSpellName.html
#Greater Invisibility             /srd/spells/greaterSpellName.html
#Heal Mass                        /srd/spells/massSpellName.html
#Heroism Greater                  /srd/spells/greaterSpellName.html
#Hold Monster Mass                /srd/spells/massSpellName.html
#Hold Person Mass                 /srd/spells/massSpellName.html
#Inflict Critical Wounds Mass     /srd/spells/massSpellName.html
#Inflict Light Wounds Mass        /srd/spells/massSpellName.html
#Inflict Moderate Wounds Mass     /srd/spells/massSpellName.html
#Inflict Serious Wounds Mass      /srd/spells/massSpellName.html
#Invisibility Mass                /srd/spells/massSpellName.html
Secret Chest                      /srd/spells/secretChest.html
Secure Shelter                    /srd/spells/secureShelter.html
Tiny Hut                          /srd/spells/tinyHut.html
Leomunds Secret Chest             /srd/spells/secretChest.html
Leomunds Secure Shelter           /srd/spells/secureShelter.html
Leomunds Tiny Hut                 /srd/spells/tinyHut.html
Leomunds Trap                     /srd/spells/trapTheSoul.html
#Magic Fang Greater               /srd/spells/greaterSpellName.html
#Magic Weapon Greater             /srd/spells/greaterSpellName.html
Acid Arrow                        /srd/spells/acidArrow.html
Melfs Acid Arrow                  /srd/spells/acidArrow.html
Mages Disjunction                 /srd/spells/magesDisjunction.html
Mages Faithful Hound              /srd/spells/magesFaithfulHound.html
Mages Lucubration                 /srd/spells/magesLucubration.html
Mages Magnificent Mansion         /srd/spells/magesMagnificentMansion.html
Mages Private Sanctum             /srd/spells/magesPrivateSanctum.html
Mages Sword                       /srd/spells/magesSword.html
Mordenkainens Disjunction         /srd/spells/magesDisjunction.html
Mordenkainens Faithful Hound      /srd/spells/magesFaithfulHound.html
Mordenkainens Lucubration         /srd/spells/magesLucubration.html
Mordenkainens Magnificent Mansion /srd/spells/magesMagnificentMansion.html
Mordenkainens Private Sanctum     /srd/spells/magesPrivateSanctum.html
Mordenkainens Sword               /srd/spells/magesSword.html
Magic Aura                        /srd/spells/magicAura.html
Nystuls Magic Aura                /srd/spells/magicAura.html
Open_Close                        /srd/spells/openClose.html
Freezing Sphere                   /srd/spells/freezingSphere.html
Resilient Sphere                  /srd/spells/resilientSphere.html
Telekinetic Sphere                /srd/spells/telekineticSphere.html
Otilukes Freezing Sphere          /srd/spells/freezingSphere.html
Otilukes Resilient Sphere         /srd/spells/resilientSphere.html
Otilukes Telekinetic Sphere       /srd/spells/telekineticSphere.html
Irresistible Dance                /srd/spells/irresistibleDance.html
Ottos Irresistible Dance          /srd/spells/irresistibleDance.html
#Owls Wisdom Mass                 /srd/spells/massSpellName.html
#Planar Ally Greater              /srd/spells/greaterSpellName.html
#Planar Ally Lesser               /srd/spells/lesserSpellName.html
#Planar Binding Greater           /srd/spells/greaterSpellName.html
#Planar Binding Lesser            /srd/spells/lesserSpellName.html
#Prying Eyes Greater              /srd/spells/greaterSpellName.html
Mnemonic Enhancer                 /srd/spells/mnemonicEnhancer.html
Telepathic Bond                   /srd/spells/telepathicBond.html
Rarys Mnemonic Enhancer           /srd/spells/mnemonicEnhancer.html
Rarys Telepathic Bond             /srd/spells/telepathicBond.html
#Reduce Person Mass               /srd/spells/massSpellName.html
Remove Blindness_Deafness         /srd/spells/removeBlindnessDeafness.html
#Restoration Greater              /srd/spells/greaterSpellName.html
#Restoration Lesser               /srd/spells/lesserSpellName.html
#Scrying Greater                  /srd/spells/greaterSpellName.html
#Shadow Conjuration Greater       /srd/spells/greaterSpellName.html
#Shadow Evocation Greater         /srd/spells/greaterSpellName.html
#Shout Greater                    /srd/spells/greaterSpellName.html
#Spell Immunity Greater           /srd/spells/greaterSpellName.html
#Suggestion Mass                  /srd/spells/massSpellName.html
Hideous Laughter                  /srd/spells/hideousLaughter.html
Tashas Hideous Laughter           /srd/spells/hideousLaughter.html
#Teleport Greater                 /srd/spells/greaterSpellName.html
Floating Disc                     /srd/spells/floatingDisk.html
Transformation                    /srd/spells/transformation.html
Tensers Floating Disc             /srd/spells/floatingDisk.html
Tensers Transformation            /srd/spells/transformation.html
